@@ -42,11 +42,13 @@ setClass("generalMatrix", representation = "VIRTUAL", contains = "compMatrix")
 setClass("symmetricMatrix",
 	 representation(uplo = "character", "VIRTUAL"),
 	 contains = "compMatrix",
+	 prototype = prototype(uplo = "U"),
 	 validity = function(object) .Call(symmetricMatrix_validate, object))
 
 setClass("triangularMatrix",
 	 representation(uplo = "character", diag = "character", "VIRTUAL"),
 	 contains = "Matrix",
+	 prototype = prototype(uplo = "U", diag = "N"),
 	 validity = function(object) .Call(triangularMatrix_validate, object))
 
 
@@ -85,6 +87,11 @@ setClass("ddenseMatrix", representation("VIRTUAL"),
 setClass("ldenseMatrix", representation("VIRTUAL"),
 	 contains = c("lMatrix", "denseMatrix"))
 
+if(FALSE) { ##--not yet--
+setClass("idenseMatrix", representation("VIRTUAL"),
+	 contains = c("iMatrix", "denseMatrix"))
+}
+
 ## Virtual class of dense, nonzero pattern matrices - rarely used, for completeness
 setClass("ndenseMatrix", representation(x = "logical", "VIRTUAL"),
 	 contains = c("nMatrix", "denseMatrix"))
@@ -92,6 +99,7 @@ setClass("ndenseMatrix", representation(x = "logical", "VIRTUAL"),
 ## diagonal: has 'diag' slot;  diag = "U"  <--> have identity matrix
 setClass("diagonalMatrix", representation(diag = "character", "VIRTUAL"),
 	 contains = "denseMatrix",
+	 ## TODO??   ^^^^^ extend just "Matrix" !?!?!? -- against DSC'07 talk (3D classes)
 	 validity = function(object) {
 	     d <- object@Dim
 	     if(d[1] != (n <- d[2])) return("matrix is not square")
@@ -117,13 +125,14 @@ setClass("TsparseMatrix", representation(i = "integer", j = "integer", "VIRTUAL"
 
 setClass("CsparseMatrix", representation(i = "integer", p = "integer", "VIRTUAL"),
 	 contains = "sparseMatrix",
+	 prototype = prototype(p = 0:0),# to be valid
          validity = function(object) .Call(Csparse_validate, object)
          )
 
 setClass("RsparseMatrix", representation(p = "integer", j = "integer", "VIRTUAL"),
-	 contains = "sparseMatrix"
-	 ## TODO:
-	 ## , validity = function(object) .Call(Rsparse_validate, object)
+	 contains = "sparseMatrix",
+	 prototype = prototype(p = 0:0),# to be valid
+	 validity = function(object) .Call(Rsparse_validate, object)
          )
 
 setClass("dsparseMatrix", representation("VIRTUAL"),
@@ -132,6 +141,11 @@ setClass("dsparseMatrix", representation("VIRTUAL"),
 setClass("lsparseMatrix", representation("VIRTUAL"),
 	 contains = c("lMatrix", "sparseMatrix"))
 
+if(FALSE) { ##--not yet--
+setClass("isparseMatrix", representation("VIRTUAL"),
+	 contains = c("lMatrix", "sparseMatrix"))
+}
+
 ## these are the "pattern" matrices for "symbolic analysis" of sparse OPs:
 setClass("nsparseMatrix", representation("VIRTUAL"),
 	 contains = c("nMatrix", "sparseMatrix"))
@@ -151,7 +165,6 @@ setClass("dgeMatrix", contains = c("ddenseMatrix", "generalMatrix"),
 ## numeric, dense, non-packed, triangular matrices
 setClass("dtrMatrix",
 	 contains = c("ddenseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N"),
 	 validity =
 	 function(object) .Call(dtrMatrix_validate, object)
 	 )
@@ -159,27 +172,19 @@ setClass("dtrMatrix",
 ## numeric, dense, packed, triangular matrices
 setClass("dtpMatrix",
 	 contains = c("ddenseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N"),
 	 validity =
 	 function(object) .Call(dtpMatrix_validate, object)
 	 )
 
 ## numeric, dense, non-packed symmetric matrices
 setClass("dsyMatrix",
-	 ## FIXME?
-	 ##> 'ddense*' before 'dge*' so it can use d* or ddense* methods
-	 ##> WITHOUT a coerce to dge* (losing symmetry)
-	 ##> gives error in crossprod() dispatch
-	 ##> contains = c("ddenseMatrix", "dgeMatrix", "symmetricMatrix"),
-	 contains = c("ddenseMatrix", "symmetricMatrix"),
-	 prototype = prototype(uplo = "U"),
+         contains = c("ddenseMatrix", "symmetricMatrix"),
 	 validity =
 	 function(object) .Call(dsyMatrix_validate, object)
 	 )
 
 ## numeric, dense, packed symmetric matrices
 setClass("dspMatrix",
-	 prototype = prototype(uplo = "U"),
 	 contains = c("ddenseMatrix", "symmetricMatrix"),
 	 validity =
 	 function(object) .Call(dspMatrix_validate, object)
@@ -206,29 +211,20 @@ setClass("lgeMatrix", contains = c("ldenseMatrix", "generalMatrix"),
 
 ## logical, dense, non-packed, triangular matrices
 setClass("ltrMatrix",
-	 contains = c("ldenseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N"))
+	 contains = c("ldenseMatrix", "triangularMatrix"))
 
 ## logical, dense, packed, triangular matrices
 setClass("ltpMatrix",
-	 contains = c("ldenseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N")
-	 ## validity: ldense*, triangular*  should suffice
-	 )
+	 contains = c("ldenseMatrix", "triangularMatrix"))
 
 ## logical, dense, non-packed symmetric matrices
 setClass("lsyMatrix",
-	 contains = c("ldenseMatrix", "symmetricMatrix"),
-	 prototype = prototype(uplo = "U")
-	 ##, validity = function(object) .Call(lsyMatrix_validate, object)
-	 )
+	 contains = c("ldenseMatrix", "symmetricMatrix"))
 
 ## logical, dense, packed symmetric matrices
 setClass("lspMatrix",
 	 contains = c("ldenseMatrix", "symmetricMatrix"),
-	 prototype = prototype(uplo = "U"),
-	 validity = function(object)
-	 .Call(dspMatrix_validate, object)
+	 validity = function(object) .Call(dspMatrix_validate, object)
 	 ## "dsp" and "lsp" have the same validate
 	 )
 
@@ -237,44 +233,35 @@ setClass("lspMatrix",
 ## logical, dense, general matrices
 setClass("ngeMatrix", contains = c("ndenseMatrix", "generalMatrix"),
 	 ## checks that length( @ x) == prod( @ Dim):
-	 validity = function(object) stopifnot(length(object@x) == prod(object@Dim))
-	 )
+	 validity = function(object)
+         stopifnot(length(object@x) == prod(object@Dim)))
 ## i.e. "ngeMatrix" cannot be packed, but "ndenseMatrix" can ..
 
 ## logical, dense, non-packed, triangular matrices
 setClass("ntrMatrix",
-	 contains = c("ndenseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N"))
+	 contains = c("ndenseMatrix", "triangularMatrix"))
 
 ## logical, dense, packed, triangular matrices
 setClass("ntpMatrix",
-	 contains = c("ndenseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N")
-	 ## validity: ndense*, triangular*  should suffice
-	 )
+	 contains = c("ndenseMatrix", "triangularMatrix"))
 
 ## logical, dense, non-packed symmetric matrices
 setClass("nsyMatrix",
-	 contains = c("ndenseMatrix", "symmetricMatrix"),
-	 prototype = prototype(uplo = "U")
-	 ## validity: ndense*, symmetric*  should suffice
-	 )
+	 contains = c("ndenseMatrix", "symmetricMatrix"))
 
 ## logical, dense, packed symmetric matrices
 setClass("nspMatrix",
 	 contains = c("ndenseMatrix", "symmetricMatrix"),
-	 prototype = prototype(uplo = "U"),
-	 validity = function(object)
-	 .Call(dspMatrix_validate, object)
+	 validity = function(object) .Call(dspMatrix_validate, object)
 	 ## "dsp" and "nsp" have the same validate
 	 )
 
 
 ## 'diagonalMatrix' already has validity checking
 ## diagonal, numeric matrices;	    "d*" has 'x' slot :
-setClass("ddiMatrix", contains = c("diagonalMatrix", "ddenseMatrix"))# or "dMatrix"
+setClass("ddiMatrix", contains = c("diagonalMatrix", "dMatrix"))# or "dMatrix"
 ## diagonal, logical matrices; "ldense*" has 'x' slot :
-setClass("ldiMatrix", contains = c("diagonalMatrix", "ldenseMatrix"))
+setClass("ldiMatrix", contains = c("diagonalMatrix", "lMatrix"))
 
 setClass("corMatrix", representation(sd = "numeric"), contains = "dpoMatrix",
 	 validity = function(object) {
@@ -306,70 +293,65 @@ setClass("dgTMatrix",
 ## numeric, sparse, triplet triangular matrices
 setClass("dtTMatrix",
 	 contains = c("TsparseMatrix", "dsparseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N"),
-	 validity = function(object) .Call(xTMatrix_validate, object)
+	 validity = function(object) .Call(tTMatrix_validate, object)
 	 )
 
-## Should not have dsTMatrix inherit from dgTMatrix because a dsTMatrix
-## is not fully stored.	 Methods for the dgTMatrix class would not
-## produce correct results even though all the slots are present.
-
-## numeric, sparse, triplet symmetric matrices
+## numeric, sparse, triplet symmetric matrices(also only store one triangle)
 setClass("dsTMatrix",
 	 contains = c("TsparseMatrix", "dsparseMatrix", "symmetricMatrix"),
-	 prototype = prototype(uplo = "U"),
-	 validity = function(object) .Call(xTMatrix_validate, object)
+	 validity = function(object) .Call(tTMatrix_validate, object)
 	 )
 
 ## numeric, sparse, sorted compressed sparse column-oriented general matrices
 setClass("dgCMatrix",
 	 contains = c("CsparseMatrix", "dsparseMatrix", "generalMatrix"),
-	 prototype = prototype(p = 0:0),# to be valid
 	 validity = function(object) .Call(xCMatrix_validate, object)
 	 )
 
+## special case: indicator rows for a factor - needs more careful definition
+#setClass("indicators", representation(levels = "character"),
+#	 contains = "dgCMatrix")
+
 ## see comments for dtTMatrix above
 ## numeric, sparse, sorted compressed sparse column-oriented triangular matrices
 setClass("dtCMatrix",
 	 contains = c("CsparseMatrix", "dsparseMatrix", "triangularMatrix"),
-	 prototype = prototype(p = 0:0, uplo = "U", diag = "N"),# to be valid
-	 validity = function(object) .Call(xCMatrix_validate, object)
+	 validity = function(object) .Call(tCMatrix_validate, object)
 	 )
 
 ## see comments for dsTMatrix above
 ## numeric, sparse, sorted compressed sparse column-oriented symmetric matrices
 setClass("dsCMatrix",
 	 contains = c("CsparseMatrix", "dsparseMatrix", "symmetricMatrix"),
-	 prototype = prototype(p = 0:0, uplo = "U"),# to be valid
-	 validity = function(object) .Call(xCMatrix_validate, object)
+	 validity = function(object) .Call(tCMatrix_validate, object)
 	 )
 
+if(FALSE) ## FIXME ??? Class of positive definite (Csparse symmetric) Matrices:
+setClass("dpCMatrix", contains = "dsCMatrix",
+	 validity = function(object) FIXME("test for pos.definite ??"))
+
 ## numeric, sparse, sorted compressed sparse row-oriented general matrices
 setClass("dgRMatrix",
 	 contains = c("RsparseMatrix", "dsparseMatrix", "generalMatrix"),
-	 prototype = prototype(p = 0:0),
-	 ##TODO: validity = function(object) .Call(dgRMatrix_validate, object)
+	 validity = function(object) .Call(xRMatrix_validate, object)
 	 )
 
 ## numeric, sparse, sorted compressed sparse row-oriented triangular matrices
 setClass("dtRMatrix",
 	 contains = c("RsparseMatrix", "dsparseMatrix", "triangularMatrix"),
-	 prototype = prototype(p = 0:0, uplo = "U", diag = "N"),# to be valid
-	 ##TODO: validity = function(object) .Call(dtRMatrix_validate, object)
-
+	 validity = function(object) .Call(tRMatrix_validate, object)
 	 )
 
 ## numeric, sparse, sorted compressed sparse row-oriented symmetric matrices
 setClass("dsRMatrix",
 	 contains = c("RsparseMatrix", "dsparseMatrix", "symmetricMatrix"),
-	 prototype = prototype(p = 0:0, uplo = "U"),# to be valid
-	 ##TODO: validity = function(object) .Call(dsRMatrix_validate, object)
+	 validity = function(object) .Call(tRMatrix_validate, object)
 	 )
 
 ##---------- logical sparse matrix classes --------------------------------
 
-## these classes are used in symbolic analysis to determine the
-## locations of non-zero entries
+## these classes are typically result of Matrix comparisons, e.g.,
+##   <..Matrix>  >= v     (and hence can have NA's)
 
 ## logical, sparse, triplet general matrices
 setClass("lgTMatrix",
@@ -380,58 +362,49 @@ setClass("lgTMatrix",
 ## logical, sparse, triplet triangular matrices
 setClass("ltTMatrix",
 	 contains = c("TsparseMatrix", "lsparseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N"),
-	 validity = function(object) .Call(xTMatrix_validate, object)
+	 validity = function(object) .Call(tTMatrix_validate, object)
 	 )
 
 ## logical, sparse, triplet symmetric matrices
 setClass("lsTMatrix",
 	 contains = c("TsparseMatrix", "lsparseMatrix", "symmetricMatrix"),
-	 prototype = prototype(uplo = "U"),
-	 validity = function(object) .Call(xTMatrix_validate, object)
+	 validity = function(object) .Call(tTMatrix_validate, object)
 	 )
 
 ## logical, sparse, sorted compressed sparse column-oriented general matrices
 setClass("lgCMatrix",
 	 contains = c("CsparseMatrix", "lsparseMatrix", "generalMatrix"),
-	 prototype = prototype(p = 0:0),# to be valid
 	 validity = function(object) .Call(xCMatrix_validate, object)
 	 )
 
 ## logical, sparse, sorted compressed sparse column-oriented triangular matrices
 setClass("ltCMatrix",
 	 contains = c("CsparseMatrix", "lsparseMatrix", "triangularMatrix"),
-	 prototype = prototype(p = 0:0, uplo = "U", diag = "N"),# to be valid
 	 validity = function(object) .Call(xCMatrix_validate, object)
 	 )
 
 ## logical, sparse, sorted compressed sparse column-oriented symmetric matrices
 setClass("lsCMatrix",
 	 contains = c("CsparseMatrix", "lsparseMatrix", "symmetricMatrix"),
-	 prototype = prototype(p = 0:0, uplo = "U"),# to be valid
 	 validity = function(object) .Call(xCMatrix_validate, object)
 	 )
 
 ## logical, sparse, sorted compressed sparse row-oriented general matrices
 setClass("lgRMatrix",
-	 representation(j = "integer", p = "integer"),
 	 contains = c("RsparseMatrix", "lsparseMatrix", "generalMatrix"),
-	 validity =
-	 function(object) .Call(lgRMatrix_validate, object)
+	 validity = function(object) .Call(xRMatrix_validate, object)
 	 )
 
 ## logical, sparse, sorted compressed sparse row-oriented triangular matrices
 setClass("ltRMatrix",
 	 contains = c("RsparseMatrix", "lsparseMatrix", "triangularMatrix"),
-	 validity =
-	 function(object) .Call(ltRMatrix_validate, object)
+	 validity = function(object) .Call(tRMatrix_validate, object)
 	 )
 
 ## logical, sparse, sorted compressed sparse row-oriented symmetric matrices
 setClass("lsRMatrix",
 	 contains = c("RsparseMatrix", "lsparseMatrix", "symmetricMatrix"),
-	 validity =
-	 function(object) .Call(lsRMatrix_validate, object)
+	 validity = function(object) .Call(tRMatrix_validate, object)
 	 )
 
 ##---------- nonzero pattern sparse matrix classes ---------------------------
@@ -448,54 +421,107 @@ setClass("ngTMatrix",
 ## nonzero pattern, sparse, triplet triangular matrices
 setClass("ntTMatrix",
 	 contains = c("TsparseMatrix", "nsparseMatrix", "triangularMatrix"),
-	 prototype = prototype(uplo = "U", diag = "N")
          ## validity: Tsparse_ and triangular*_validate should be enough
 	 )
 
 ## nonzero pattern, sparse, triplet symmetric matrices
 setClass("nsTMatrix",
 	 contains = c("TsparseMatrix", "nsparseMatrix", "symmetricMatrix"),
-	 prototype = prototype(uplo = "U")
          ## validity: Tsparse_ and symmetric*_validate should be enough
 	 )
 
 ## nonzero pattern, sparse, sorted compressed column-oriented general matrices
 setClass("ngCMatrix",
 	 contains = c("CsparseMatrix", "nsparseMatrix", "generalMatrix"),
-	 prototype = prototype(p = 0:0)# to be valid
          ## validity: Csparse_validate should be enough
 	 )
 
 ## nonzero pattern, sparse, sorted compressed column-oriented triangular matrices
 setClass("ntCMatrix",
 	 contains = c("CsparseMatrix", "nsparseMatrix", "triangularMatrix"),
-	 prototype = prototype(p = 0:0, uplo = "U", diag = "N")# to be valid
          ## validity: Csparse_ and triangular*_validate should be enough
 	 )
 
 ## nonzero pattern, sparse, sorted compressed column-oriented symmetric matrices
 setClass("nsCMatrix",
 	 contains = c("CsparseMatrix", "nsparseMatrix", "symmetricMatrix"),
-	 prototype = prototype(p = 0:0, uplo = "U")# to be valid
          ## validity: Csparse_ and symmetric*_validate should be enough
 	 )
 
 ## nonzero pattern, sparse, sorted compressed row-oriented general matrices
 setClass("ngRMatrix",
-	 representation(j = "integer", p = "integer"),
-	 contains = c("RsparseMatrix", "nsparseMatrix", "generalMatrix")
+	 contains = c("RsparseMatrix", "nsparseMatrix", "generalMatrix"),
 	 )
 
 ## nonzero pattern, sparse, sorted compressed row-oriented triangular matrices
 setClass("ntRMatrix",
-	 contains = c("RsparseMatrix", "nsparseMatrix", "triangularMatrix")
+	 contains = c("RsparseMatrix", "nsparseMatrix", "triangularMatrix"),
 	 )
 
 ## nonzero pattern, sparse, sorted compressed row-oriented symmetric matrices
 setClass("nsRMatrix",
-	 contains = c("RsparseMatrix", "nsparseMatrix", "symmetricMatrix")
+	 contains = c("RsparseMatrix", "nsparseMatrix", "symmetricMatrix"),
 	 )
 
+if(FALSE) { ##--not yet--
+
+##---------- integer sparse matrix classes --------------------------------
+
+## integer, sparse, triplet general matrices
+setClass("igTMatrix",
+	 contains = c("TsparseMatrix", "isparseMatrix", "generalMatrix"),
+	 validity = function(object) .Call(xTMatrix_validate, object)
+	 )
+
+## integer, sparse, triplet triangular matrices
+setClass("itTMatrix",
+	 contains = c("TsparseMatrix", "isparseMatrix", "triangularMatrix"),
+	 validity = function(object) .Call(tTMatrix_validate, object)
+	 )
+
+## integer, sparse, triplet symmetric matrices
+setClass("isTMatrix",
+	 contains = c("TsparseMatrix", "isparseMatrix", "symmetricMatrix"),
+	 validity = function(object) .Call(tTMatrix_validate, object)
+	 )
+
+## integer, sparse, sorted compressed sparse column-oriented general matrices
+setClass("igCMatrix",
+	 contains = c("CsparseMatrix", "isparseMatrix", "generalMatrix"),
+	 validity = function(object) .Call(xCMatrix_validate, object)
+	 )
+
+## integer, sparse, sorted compressed sparse column-oriented triangular matrices
+setClass("itCMatrix",
+	 contains = c("CsparseMatrix", "isparseMatrix", "triangularMatrix"),
+	 validity = function(object) .Call(xCMatrix_validate, object)
+	 )
+
+## integer, sparse, sorted compressed sparse column-oriented symmetric matrices
+setClass("isCMatrix",
+	 contains = c("CsparseMatrix", "isparseMatrix", "symmetricMatrix"),
+	 validity = function(object) .Call(xCMatrix_validate, object)
+	 )
+
+## integer, sparse, sorted compressed sparse row-oriented general matrices
+setClass("igRMatrix",
+	 contains = c("RsparseMatrix", "isparseMatrix", "generalMatrix"),
+	 validity = function(object) .Call(xRMatrix_validate, object)
+	 )
+
+## integer, sparse, sorted compressed sparse row-oriented triangular matrices
+setClass("itRMatrix",
+	 contains = c("RsparseMatrix", "isparseMatrix", "triangularMatrix"),
+	 validity = function(object) .Call(tRMatrix_validate, object)
+	 )
+
+## integer, sparse, sorted compressed sparse row-oriented symmetric matrices
+setClass("isRMatrix",
+	 contains = c("RsparseMatrix", "isparseMatrix", "symmetricMatrix"),
+	 validity = function(object) .Call(tRMatrix_validate, object)
+	 )
+}##--not yet--
+
 ##-------------------- permutation ----------------------------------------
 
 setClass("pMatrix", representation(perm = "integer"),
@@ -602,6 +628,27 @@ setClass("css_QR", representation(Pinv = "integer", Q = "integer",
 setClass("css_LU", representation(Q = "integer", nz = "integer"))
 }
 
+##-- Schur ---
+
+## non-"Matrix" Class 1  --- For Eigen values:
+setClassUnion("number", members = c("numeric", "complex"))
+
+setClass("Schur", contains = "MatrixFactorization",
+	 representation(T = "Matrix", # <- "block-triangular"; maybe triangular
+			Q = "Matrix", EValues = "number"),
+	 validity = function(object) {
+	     dim <- object@Dim
+	     if((n <- dim[1]) != dim[2])
+		 "'Dim' slot is not (n,n)"
+	     else if(any(dim(object@T) != n))
+		 "'dim(T)' is incorrect"
+	     else if(any(dim(object@Q) != n))
+		 "'dim(Q)' is incorrect"
+	     else if(length(object@EValues) != n)
+		 "'EValues' is not of correct length"
+	     else TRUE
+	 })
+
 
 ### Class Union :  no inheritance, but is(*, <class>) :
 
@@ -617,15 +664,57 @@ setClass("css_LU", representation(Q = "integer", nz = "integer"))
 
 ## --- "General" (not Matrix at all) ----
 
-## for 'i' in x[i] or A[i,] :
+## for 'i' in x[i] or A[i,] : (numeric = {double, integer})
 setClassUnion("index", members =  c("numeric", "logical", "character"))
 
-### for 'value' in   x[..] <- value :
-setClassUnion("replValue", members =  c("numeric", "logical"))
-
+## "atomic vectors" (-> ?is.atomic ) --
+## ---------------  those that we want to convert from old-style "matrix"
+setClassUnion("atomicVector", ## "double" is not needed, and not liked by some
+	      members = c("logical", "integer", "numeric",
+			  "complex", "raw", "character"))
 
 ## --- Matrix - related (but not "Matrix" nor "Decomposition/Factorization):
 
+### for 'value' in  x[..] <- value hence for all "contents" of our Matrices:
+setClassUnion("replValue", members = c("numeric", "logical", "complex", "raw"))
+
+### Sparse Vectors ---- here use 1-based indexing ! -----------
+setClass("sparseVector",
+         representation(length = "integer", i = "integer", "VIRTUAL"),
+         validity = function(object) {
+             n <- object@length
+             if(any(object@i < 1L) || any(object@i > n))
+                 sprintf("'i' must be in 1:%d", n)
+             else TRUE
+         })
+.validXspVec <- function(object) {
+	     n <- object@length
+	     if(length(object@i) != length(object@x))
+		 "'i' and 'x' differ in length"
+	     else TRUE
+}
+setClass("dsparseVector",
+	 representation(x = "numeric"), contains = "sparseVector",
+	 validity = .validXspVec)
+setClass("isparseVector",
+	 representation(x = "integer"), contains = "sparseVector",
+	 validity = .validXspVec)
+setClass("lsparseVector",
+	 representation(x = "logical"), contains = "sparseVector",
+	 validity = .validXspVec)
+setClass("zsparseVector",
+	 representation(x = "complex"), contains = "sparseVector",
+	 validity = .validXspVec)
+## nsparse has no new slot: 'i' just contains the locations!
+setClass("nsparseVector", contains = "sparseVector")
+
+setClassUnion("xsparseVector", ## those sparseVector's with an 'x' slot
+              c("dsparseVector",
+                "isparseVector",
+                "lsparseVector",
+                "zsparseVector"))
+
+
 setClass("determinant",
 	 representation(modulus = "numeric",
 			logarithm = "logical",
