@@ -10,8 +10,7 @@ setAs("dgeMatrix", "matrix",
 
 ##  "Arith" is in ./Ops.R
 
-setMethod("Math",
-	  signature(x = "dgeMatrix"),
+setMethod("Math", signature(x = "dgeMatrix"),
 	  function(x) {
 	      x@x <- callGeneric(x@x)
 	      x
@@ -38,14 +37,13 @@ setMethod("norm", signature(x = "dgeMatrix", type = "character"),
 	  function(x, type, ...) .Call(dgeMatrix_norm, x, type),
 	  valueClass = "numeric")
 
-setMethod("rcond", signature(x = "dgeMatrix", type = "missing"),
-	  function(x, type, ...) rcond(x, type = "O", ...))
-setMethod("rcond", signature(x = "dgeMatrix", type = "character"),
-	  function(x, type, ...)  {
+setMethod("rcond", signature(x = "dgeMatrix", norm = "missing"),
+	  function(x, norm, ...) rcond(x, norm = "O", ...))
+setMethod("rcond", signature(x = "dgeMatrix", norm = "character"),
+	  function(x, norm, ...)  {
 	      if({d <- dim(x); d[1] == d[2]})
-		  .Call(dgeMatrix_rcond, x, type)
-	      else
-		  rcond(qr.R(qr(x)), type=type)
+		  .Call(dgeMatrix_rcond, x, norm)
+	      else rcond(qr.R(qr(if(d[1] < d[2]) t(x) else x)), norm=norm, ...)
 	  },
 	  valueClass = "numeric")
 
@@ -54,8 +52,11 @@ setMethod("norm", signature(x = "matrix", type = "character"),
 	  function(x, type, ...) .Call(dgeMatrix_norm, as(x,"dgeMatrix"), type),
 	  valueClass = "numeric")
 
-setMethod("rcond", signature(x = "matrix", type = "character"),
-	  function(x, type, ...) rcond(as(x,"dgeMatrix"), type))
+## for now:
+if(!existsFunction("rcond", where=baseenv()))## later: (exists(getRversion() < "2.7.0")
+## afterwards, this should continue using base::rcond
+setMethod("rcond", signature(x = "matrix", norm = "character"),
+	  function(x, norm, ...) rcond(as(x,"dgeMatrix"), norm=norm, ...))
 
 
 setMethod("t", signature(x = "dgeMatrix"), t_geMatrix)
@@ -76,7 +77,7 @@ setMethod("tcrossprod", signature(x = "matrix", y = "missing"),
 	  valueClass = "dpoMatrix")
 
 setMethod("tcrossprod", signature(x = "numeric", y = "missing"),
-	  function(x, y = NULL) callGeneric(as.matrix(as.double(x))))
+	  function(x, y = NULL) tcrossprod(as.matrix(as.double(x))))
 }
 
 ## crossprod (x,y)
@@ -92,10 +93,10 @@ setMethod("crossprod", signature(x = "dgeMatrix", y = "numeric"),
 	  .Call(dgeMatrix_matrix_crossprod, x, as.matrix(as.double(y)), FALSE),
 	  valueClass = "dgeMatrix")
 setMethod("crossprod", signature(x = "matrix", y = "dgeMatrix"),
-	  function(x, y = NULL) callGeneric(as(x, "dgeMatrix"), y),
+	  function(x, y = NULL) crossprod(as(x, "dgeMatrix"), y),
 	  valueClass = "dgeMatrix")
 setMethod("crossprod", signature(x = "numeric", y = "dgeMatrix"),
-	  function(x, y = NULL) callGeneric(as.matrix(as.double(x)), y),
+	  function(x, y = NULL) crossprod(as.matrix(as.double(x)), y),
 	  valueClass = "dgeMatrix")
 
 ## tcrossprod (x,y)
@@ -111,10 +112,10 @@ setMethod("tcrossprod", signature(x = "dgeMatrix", y = "numeric"),
 	  .Call(dgeMatrix_matrix_crossprod, x, rbind(as.double(y)), TRUE),
 	  valueClass = "dgeMatrix")
 setMethod("tcrossprod", signature(x = "matrix", y = "dgeMatrix"),
-	  function(x, y = NULL) callGeneric(as(x, "dgeMatrix"), y),
+	  function(x, y = NULL) tcrossprod(as(x, "dgeMatrix"), y),
 	  valueClass = "dgeMatrix")
 setMethod("tcrossprod", signature(x = "numeric", y = "dgeMatrix"),
-	  function(x, y = NULL) callGeneric(rbind(as.double(x)), y),
+	  function(x, y = NULL) tcrossprod(rbind(as.double(x)), y),
 	  valueClass = "dgeMatrix")
 
 ## %*% methods
@@ -145,7 +146,7 @@ setMethod("%*%", signature(x = "numeric", y = "dgeMatrix"),
 setMethod("diag", signature(x = "dgeMatrix"),
 	  function(x, nrow, ncol) .Call(dgeMatrix_getDiag, x))
 
-setMethod("chol", signature(x = "dgeMatrix", pivot = "ANY"), cholMat)
+setMethod("chol", signature(x = "dgeMatrix"), cholMat)
 
 setMethod("solve", signature(a = "dgeMatrix", b = "missing"),
 	  function(a, b, ...) .Call(dgeMatrix_solve, a),
@@ -176,9 +177,6 @@ setMethod("determinant", signature(x = "dgeMatrix", logarithm = "logical"),
 	  function(x, logarithm, ...)
 	  .Call(dgeMatrix_determinant, x, logarithm))
 
-
-setMethod("expm", signature(x = "dgeMatrix"),
-	  function(x) .Call(dgeMatrix_exp, x),
-	  valueClass = "dgeMatrix")
+##-> ./expm.R  for expm()
 
 ##-> ./colSums.R  for colSums,... rowMeans
