@@ -165,11 +165,13 @@ setAs("TsparseMatrix", "graphNEL", Tsp2grNEL)
 
 setMethod("[", signature(x = "sparseMatrix", i = "index", j = "missing",
 			 drop = "logical"),
-	  function (x, i, drop) {
+	  function (x, i,j, ..., drop) {
 	      cld <- getClassDef(class(x))
-	      if(!extends(cld, "generalMatrix")) x <- as(x, "generalMatrix")
-	      viaCl <- paste(.M.kind(x, cld), "gTMatrix", sep='')
-	      x <- callGeneric(x = as(x, viaCl), i=i, drop=drop)
+##> why should this be needed; can still happen in <Tsparse>[..]:
+##>	      if(!extends(cld, "generalMatrix")) x <- as(x, "generalMatrix")
+##	      viaCl <- paste(.M.kind(x, cld), "gTMatrix", sep='')
+	      x <- as(x, "TsparseMatrix")[i, , drop=drop]
+##simpler than x <- callGeneric(x = as(x, "TsparseMatrix"), i=i, drop=drop)
 	      ## try_as(x, c(cl, sub("T","C", viaCl)))
 	      if(is(x, "Matrix") && extends(cld, "CsparseMatrix"))
 		  as(x, "CsparseMatrix") else x
@@ -177,11 +179,14 @@ setMethod("[", signature(x = "sparseMatrix", i = "index", j = "missing",
 
 setMethod("[", signature(x = "sparseMatrix", i = "missing", j = "index",
 			 drop = "logical"),
-	  function (x, j, drop) {
+	  function (x,i,j, ..., drop) {
 	      cld <- getClassDef(class(x))
-	      if(!extends(cld, "generalMatrix")) x <- as(x, "generalMatrix")
-	      viaCl <- paste(.M.kind(x, cld), "gTMatrix", sep='')
-	      x <- callGeneric(x = as(x, viaCl), j=j, drop=drop)
+##> why should this be needed; can still happen in <Tsparse>[..]:
+##>	      if(!extends(cld, "generalMatrix")) x <- as(x, "generalMatrix")
+##	      viaCl <- paste(.M.kind(x, cld), "gTMatrix", sep='')
+
+	      x <- as(x, "TsparseMatrix")[, j, drop=drop]
+##simpler than x <- callGeneric(x = as(x, "TsparseMatrix"), j=j, drop=drop)
 	      if(is(x, "Matrix") && extends(cld, "CsparseMatrix"))
 		  as(x, "CsparseMatrix") else x
 	  })
@@ -191,13 +196,14 @@ setMethod("[", signature(x = "sparseMatrix",
 	  function (x, i, j, ..., drop) {
 	      cld <- getClassDef(class(x))
 	      ## be smart to keep symmetric indexing of <symm.Mat.> symmetric:
-	      doSym <- (extends(cld, "symmetricMatrix") &&
-			length(i) == length(j) && all(i == j))
-	      if(!doSym && !extends(cld, "generalMatrix"))
-		  x <- as(x, "generalMatrix")
-	      viaCl <- paste(.M.kind(x, cld),
-			     if(doSym) "sTMatrix" else "gTMatrix", sep='')
-	      x <- callGeneric(x = as(x, viaCl), i=i, j=j, drop=drop)
+##>	      doSym <- (extends(cld, "symmetricMatrix") &&
+##>			length(i) == length(j) && all(i == j))
+##> why should this be needed; can still happen in <Tsparse>[..]:
+##>	      if(!doSym && !extends(cld, "generalMatrix"))
+##>		  x <- as(x, "generalMatrix")
+##	      viaCl <- paste(.M.kind(x, cld),
+##			     if(doSym) "sTMatrix" else "gTMatrix", sep='')
+	      x <- as(x, "TsparseMatrix")[i, j, drop=drop]
 	      if(is(x, "Matrix") && extends(cld, "CsparseMatrix"))
 		  as(x, "CsparseMatrix") else x
 	  })
@@ -215,7 +221,7 @@ setMethod("Math",
 	  signature(x = "sparseMatrix"),
 	  function(x) callGeneric(as(x, "CsparseMatrix")))
 
-## further group methods -> see ./Ops.R
+## further group methods -> see ./Ops.R {"Summary": ./dMatrix.R }
 
 
 
@@ -224,7 +230,7 @@ setMethod("Math",
 ## FIXME(?) -- ``merge this'' (at least ``synchronize'') with
 ## - - -   prMatrix() from ./Auxiliaries.R
 ## FIXME: prTriang() in ./Auxiliaries.R  should also get  align = "fancy"
-## --> help for this is currently (rudimentary) in ../man/sparseMatrix-class.Rd
+##
 printSpMatrix <- function(x, digits = getOption("digits"),
 		       maxp = getOption("max.print"), zero.print = ".",
 		       col.names, note.dropping.colnames = TRUE,
@@ -232,6 +238,7 @@ printSpMatrix <- function(x, digits = getOption("digits"),
 {
     cl <- getClassDef(class(x))
     stopifnot(extends(cl, "sparseMatrix"))
+    validObject(x) # have seen seg.faults for invalid objects
     d <- dim(x)
     if(prod(d) > maxp) { # "Large" => will be "cut"
         ## only coerce to dense that part which won't be cut :
@@ -244,7 +251,7 @@ printSpMatrix <- function(x, digits = getOption("digits"),
     logi <- extends(cl,"lsparseMatrix") || extends(cl,"nsparseMatrix")
     if(logi)
 	cx <- array("N", dim(m), dimnames=dn)
-    else { ## numeric (or --not yet-- complex):
+    else { ## numeric (or --not yet implemented-- complex):
 	cx <- apply(m, 2, format)
 	if(is.null(dim(cx))) {# e.g. in	1 x 1 case
 	    dim(cx) <- dim(m)
@@ -287,10 +294,10 @@ printSpMatrix <- function(x, digits = getOption("digits"),
 	## show only "structural" zeros as 'zero.print', not all of them..
 	## -> cannot use 'm'
         d <- dim(cx)
-	ne <- length(iN0 <- 1L + encodeInd(non0ind(x, cl), nr = d[1]))
+	ne <- length(iN0 <- 1L + .Call(m_encodeInd, non0ind(x, cl), di = d))
 	if(0 < ne && ne < prod(d)) {
 	    align <- match.arg(align)
-	    if(align == "fancy") {
+	    if(align == "fancy" && !is.integer(m)) {
 		fi <- apply(m, 2, format.info) ## fi[3,] == 0  <==> not expo.
 		## now 'format' the zero.print by padding it with ' ' on the right:
 		## case 1: non-exponent:  fi[2,] + as.logical(fi[2,] > 0)
@@ -323,57 +330,81 @@ printSpMatrix <- function(x, digits = getOption("digits"),
     ## right = TRUE : cheap attempt to get better "." alignment
     print(cx, quote = FALSE, right = TRUE, max = maxp)
     invisible(x)
-}
+} ## printSpMatrix()
+
+printSpMatrix2 <- function(x, digits = getOption("digits"),
+                           maxp = getOption("max.print"), zero.print = ".",
+                           col.names, note.dropping.colnames = TRUE,
+                           suppRows = NULL, suppCols = NULL,
+                           col.trailer = if(suppCols) "......" else "",
+                           align = c("fancy", "right"))
+{
+    d <- dim(x)
+    if((identical(suppRows,FALSE) && identical(suppCols, FALSE)) ||
+       (!isTRUE(suppRows) && !isTRUE(suppCols) && prod(d) <= maxp))
+    {
+        if(missing(col.trailer) && is.null(suppCols))
+            suppCols <- FALSE # for 'col.trailer'
+        printSpMatrix(x, digits=digits, maxp=maxp,
+                      zero.print=zero.print, col.names=col.names,
+                      note.dropping.colnames=note.dropping.colnames,
+                      col.trailer=col.trailer, align=align)
+    } else { ## d[1] > maxp / d[2] >= nr : -- this needs [,] working:
+        nR <- d[1] ## nrow
+        useW <- getOption("width") - (format.info(nR)[1] + 3+1)
+        ##                           space for "[<last>,] "
+
+        ## --> suppress rows and/or columns in printing ...
+
+        if(is.null(suppCols)) suppCols <- (d[2] * 2 > useW)
+        nc <- if(suppCols) (useW - (1 + nchar(col.trailer))) %/% 2 else d[2]
+        nr <- maxp %/% nc
+        if(is.null(suppRows)) suppRows <- (nr < nR)
+
+        sTxt <- c("in show(); maybe adjust 'options(max.print= *)'",
+                  "\n ..............................\n")
+        if(suppRows) {
+            if(suppCols)
+                x <- x[ , 1:nc, drop = FALSE]
+            n2 <- ceiling(nr / 2)
+            printSpMatrix(x[seq_len(min(nR, max(1, n2))), , drop=FALSE],
+                          digits=digits, maxp=maxp,
+                          zero.print=zero.print, col.names=col.names,
+                          note.dropping.colnames=note.dropping.colnames,
+                          col.trailer = col.trailer, align=align)
+            cat("\n ..............................",
+                "\n ........suppressing rows ", sTxt, "\n", sep='')
+            ## tail() automagically uses "[..,]" rownames:
+            printSpMatrix(tail(x, max(1, nr-n2)),
+                          digits=digits, maxp=maxp,
+                          zero.print=zero.print, col.names=col.names,
+                          note.dropping.colnames=note.dropping.colnames,
+                          col.trailer = col.trailer, align=align)
+        }
+        else if(suppCols) {
+            printSpMatrix(x[ , 1:nc , drop = FALSE],
+                          digits=digits, maxp=maxp,
+                          zero.print=zero.print, col.names=col.names,
+                          note.dropping.colnames=note.dropping.colnames,
+                          col.trailer = col.trailer, align=align)
+            cat("\n .....suppressing columns ", sTxt, sep='')
+        }
+        else stop("logic programming error in printSpMatrix2(), please report")
+
+        invisible(x)
+    }
+} ## printSpMatrix2 ()
 
 setMethod("print", signature(x = "sparseMatrix"), printSpMatrix)
 
 setMethod("show", signature(object = "sparseMatrix"),
-   function(object) {
-       d <- dim(object)
-       cl <- class(object)
-       cat(sprintf('%d x %d sparse Matrix of class "%s"\n', d[1], d[2], cl))
-       maxp <- getOption("max.print")
-       if(prod(d) <= maxp)
-	   printSpMatrix(object, maxp = maxp)
-       else { ## d[1] > maxp / d[2] >= nr : -- this needs [,] working:
-
-	   nR <- d[1] # nrow
-           useW <- getOption("width") - (format.info(nR)[1] + 3+1)
-           ##                           space for "[<last>,] "
-
-           ## --> suppress rows and/or columns in printing ...
-
-           suppCols <- (d[2] * 2 > useW)
-           nc <- if(suppCols) (useW - (1 + 6)) %/% 2 else d[2]
-           ##                          sp+ col.trailer
-           col.trailer <- if(suppCols) "......" else ""
-	   nr <- maxp %/% nc
-           suppRows <- (nr < nR)
-           if(suppRows) {
-	       if(suppCols)
-		   object <- object[ , 1:nc, drop = FALSE]
-	       n2 <- ceiling(nr / 2)
-	       printSpMatrix(object[seq_len(min(nR, max(1, n2))), , drop=FALSE],
-                             col.trailer = col.trailer)
-	       cat("\n ..............................",
-		   "\n ..........suppressing rows in show(); maybe adjust 'options(max.print= *)'",
-		   "\n ..............................\n\n", sep='')
-	       ## tail() automagically uses "[..,]" rownames:
-	       printSpMatrix(tail(object, max(1, nr-n2)),
-                             col.trailer = col.trailer)
-	   }
-	   else if(suppCols) {
-	       printSpMatrix(object[ , 1:nc , drop = FALSE],
-                             col.trailer = col.trailer)
-
-	       cat("\n .....suppressing columns in show(); maybe adjust 'options(max.print= *)'",
-		   "\n ..............................\n", sep='')
-	   }
-           else stop("logic programming error in printSpMatrix(), please report")
-
-           invisible(object)
-       }
-   })
+          function(object) {
+              d <- dim(object)
+              cl <- class(object)
+              cat(sprintf('%d x %d sparse Matrix of class "%s"\n',
+                          d[1], d[2], cl))
+              printSpMatrix2(object)
+          })
 
 
 ## For very large and very sparse matrices,  the above show()
@@ -384,10 +415,12 @@ setMethod("summary", signature(object = "sparseMatrix"),
 	      d <- dim(object)
 	      T <- as(object, "TsparseMatrix")
 	      ## return a data frame (int, int,	 {double|logical|...})	:
-	      r <- data.frame(i = T@i + 1L, j = T@j + 1L, x = T@x)
+	      r <- if(is(object,"nsparseMatrix"))
+		  data.frame(i = T@i + 1L, j = T@j + 1L)
+	      else data.frame(i = T@i + 1L, j = T@j + 1L, x = T@x)
 	      attr(r, "header") <-
 		  sprintf('%d x %d sparse Matrix of class "%s", with %d entries',
-			  d[1], d[2], class(object), nnzero(object))
+			  d[1], d[2], class(object), length(T@i))
 	      ## use ole' S3 technology for such a simple case
 	      class(r) <- c("sparseSummary", class(r))
 	      r
@@ -400,31 +433,38 @@ print.sparseSummary <- function (x, ...) {
 }
 
 setMethod("isSymmetric", signature(object = "sparseMatrix"),
-	  function(object, tol = 100*.Machine$double.eps) {
+	  function(object, tol = 100*.Machine$double.eps, ...) {
 	      ## pretest: is it square?
 	      d <- dim(object)
 	      if(d[1] != d[2]) return(FALSE)
-	      ## else slower test
+
+	      ## else slower test using t()  --
+
+	      ## FIXME (for tol = 0): use cholmod_symmetry(A, 1, ...)
+	      ##        for tol > 0   should modify  cholmod_symmetry(..) to work with tol
+
+	      ## or slightly simpler, rename and export	 is_sym() in ../src/cs_utils.c
+
+
 	      if (is(object, "dMatrix"))
 		  ## use gC; "T" (triplet) is *not* unique!
 		  isTRUE(all.equal(.as.dgC.0.factors(  object),
-				   .as.dgC.0.factors(t(object)), tol = tol))
+				   .as.dgC.0.factors(t(object)),
+				   tol = tol, ...))
 	      else if (is(object, "lMatrix"))
 		  ## test for exact equality; FIXME(?): identical() too strict?
-		  identical(as(object, "lgCMatrix"),
+		  identical(as(	 object,  "lgCMatrix"),
 			    as(t(object), "lgCMatrix"))
 	      else if (is(object, "nMatrix"))
 		  ## test for exact equality; FIXME(?): identical() too strict?
-		  identical(as(object, "ngCMatrix"),
+		  identical(as(	 object,  "ngCMatrix"),
 			    as(t(object), "ngCMatrix"))
 	      else stop("not yet implemented")
 	  })
 
 
-## These two are not (yet?) exported:
-setMethod("isTriangular", signature(object = "sparseMatrix"),
-	  function(object, upper = NA)
-              isTriC(as(object, "CsparseMatrix"), upper))
+setMethod("isTriangular", signature(object = "CsparseMatrix"), isTriC)
+setMethod("isTriangular", signature(object = "TsparseMatrix"), isTriT)
 
 setMethod("isDiagonal", signature(object = "sparseMatrix"),
 	  function(object) {
@@ -436,6 +476,14 @@ setMethod("isDiagonal", signature(object = "sparseMatrix"),
 	  })
 
 
+setMethod("determinant", signature(x = "sparseMatrix", logarithm = "missing"),
+	  function(x, logarithm, ...)
+	  determinant(x, logarithm = TRUE, ...))
+setMethod("determinant", signature(x = "sparseMatrix", logarithm = "logical"),
+	  function(x, logarithm = TRUE, ...)
+	  determinant(as(x,"dsparseMatrix"), logarithm, ...))
+
+
 setMethod("diag", signature(x = "sparseMatrix"),
 	  function(x, nrow, ncol = n) diag(as(x, "CsparseMatrix")))
 
@@ -453,9 +501,6 @@ setMethod("dim<-", signature(x = "sparseMatrix", value = "ANY"),
 
 setMethod("norm", signature(x = "sparseMatrix", type = "character"),
 	  function(x, type, ...) {
-## as(*, "dsparseMatrix") fails e.g. for "lgT*", but why use it anyway?
-## 	      if(!is(x, "dsparseMatrix"))
-## 		  x <- as(x, "dsparseMatrix")
 	      type <- toupper(substr(type[1], 1, 1))
 	      switch(type,  ##  max(<empty>, 0)  |-->  0
 		     "O" = ,
@@ -467,17 +512,54 @@ setMethod("norm", signature(x = "sparseMatrix", type = "character"),
 		     stop("invalid 'type'"))
 	  })
 
-setMethod("rcond", signature(x = "sparseMatrix", type = "character"),
-	  function(x, type, ...) {
+setMethod("rcond", signature(x = "sparseMatrix", norm = "character"),
+	  function(x, norm, ...) {
 	      d <- dim(x)
+              ## FIXME: qr.R(qr(.)) warns about differing R (permutation!)
+              ##        really fix qr.R() *or* go via dense in any cases
 	      rcond(if(d[1] == d[2]) {
-			warning("rcond(.) via  sparse -> dense	coercion")
+			warning("rcond(.) via sparse -> dense coercion")
 			as(x, "denseMatrix")
 		    } else if(d[1] > d[2]) qr.R(qr(x)) else qr.R(qr(t(x))),
-		    type = type)
+		    norm = norm, ...)
+	  })
+
+setMethod("cov2cor", signature(V = "sparseMatrix"),
+	  function(V) {
+	      ## like stats::cov2cor() but making sure all matrices stay sparse
+	      p <- (d <- dim(V))[1]
+	      if (p != d[2])
+		  stop("'V' is not a *square* matrix")
+	      if(!is(V, "dMatrix"))
+		  V <- as(V, "dMatrix")# actually "dsparseMatrix"
+	      Is <- sqrt(1/diag(V))
+	      if (any(!is.finite(Is))) ## original had 0 or NA
+		  warning("diag(.) had 0 or NA entries; non-finite result is doubtful")
+	      ## TODO: if  <diagonal> %*% <sparse> was implemented more efficiently
+	      ##       we'd rather use that!
+	      Is <- as(Diagonal(x = Is), "sparseMatrix")
+	      r <- Is %*% V %*% Is
+	      r[cbind(1L:p,1L:p)] <- 1 # exact in diagonal
+	      as(r, "symmetricMatrix")
+	  })
+
+setMethod("is.na", signature(x = "sparseMatrix"),## NB: nsparse* have own method!
+	  function(x) {
+	      if(any((inax <- is.na(x@x)))) {
+		  cld <- getClassDef(class(x))
+		  if(extends(cld, "triangularMatrix") && x@diag == "U")
+		      inax <- is.na((x <- .diagU2N(x, cld))@x)
+		  r <- as(x, "lMatrix") # will be "lsparseMatrix" - *has* x slot
+		  r@x <- inax
+                  if(!extends(cld, "CsparseMatrix"))
+                      r <- as(r, "CsparseMatrix")
+		  as(.Call(Csparse_drop, r, 0), "nMatrix") # a 'pattern matrix
+	      }
+	      else is.na_nsp(x)
 	  })
 
 
+### Keep this namespace-hidden: Would need to return a classed object
 lm.fit.sparse <-
 function(x, y, offset = NULL, method = c("qr", "cholesky"),
          tol = 1e-7, singular.ok = TRUE, transpose = FALSE, ...)
@@ -524,3 +606,74 @@ fac2sparse <- function(from, to = c("d","i","l","n","z"))
 }
 
 setAs("factor", "sparseMatrix", function(from) fac2sparse(from, to = "d"))
+
+## xtabs returning a sparse matrix.  This is cut'n'paste
+## of xtabs() in <Rsrc>/src/library/stats/R/xtabs.R ;
+## with the new argument 'sparse'
+xtabs <- function(formula = ~., data = parent.frame(), subset, sparse = FALSE,
+		  na.action, exclude = c(NA, NaN), drop.unused.levels = FALSE)
+{
+    if (missing(formula) && missing(data))
+	stop("must supply either 'formula' or 'data'")
+    if(!missing(formula)){
+	## We need to coerce the formula argument now, but model.frame
+	## will coerce the original version later.
+	formula <- as.formula(formula)
+	if (!inherits(formula, "formula"))
+	    stop("'formula' missing or incorrect")
+    }
+    if (any(attr(terms(formula, data = data), "order") > 1))
+	stop("interactions are not allowed")
+    m <- match.call(expand.dots = FALSE)
+    if (is.matrix(eval(m$data, parent.frame())))
+	m$data <- as.data.frame(data)
+    m$... <- m$exclude <- m$drop.unused.levels <- m$sparse <- NULL
+    m[[1]] <- as.name("model.frame")
+    mf <- eval(m, parent.frame())
+    if(length(formula) == 2) {
+	by <- mf
+	y <- NULL
+    }
+    else {
+	i <- attr(attr(mf, "terms"), "response")
+	by <- mf[-i]
+	y <- mf[[i]]
+    }
+    by <- lapply(by, function(u) {
+	if(!is.factor(u)) u <- factor(u, exclude = exclude)
+	u[ , drop = drop.unused.levels]
+    })
+    if(!sparse) { ## identical to stats::xtabs
+	x <-
+	    if(is.null(y))
+		do.call("table", by)
+	    else if(NCOL(y) == 1)
+		tapply(y, by, sum)
+	    else {
+		z <- lapply(as.data.frame(y), tapply, by, sum)
+		array(unlist(z),
+		      dim = c(dim(z[[1]]), length(z)),
+		      dimnames = c(dimnames(z[[1]]), list(names(z))))
+	    }
+	x[is.na(x)] <- 0
+	class(x) <- c("xtabs", "table")
+	attr(x, "call") <- match.call()
+	x
+
+    } else { ## sparse
+	if (length(by) != 2)
+	    stop("xtabs(*, sparse=TRUE) applies only to two-way tables")
+	rows <- by[[1]]
+	cols <- by[[2]]
+	rl <- levels(rows)
+	cl <- levels(cols)
+	if (is.null(y))
+	    y <- rep.int(1, length(rows))
+	as(new("dgTMatrix",
+	       i = as.integer(rows) - 1L,
+	       j = as.integer(cols) - 1L,
+	       x = as.double(y),
+	       Dim = c(length(rl), length(cl)),
+	       Dimnames = list(rl, cl)), "CsparseMatrix")
+    }
+}
